#!/usr/bin/env node

const path = require('path');
const fs = require('fs');

const chalk = require('chalk');
const yaml = require('js-yaml');
const archiver = require('archiver');

const manifestPath = path.resolve('manifest.yaml');
const staticFilesPath = path.resolve('build');
const bundlePath = path.resolve('bundle.zip');

checkManifest();
checkStaticFiles();
bundleApp();

function checkManifest() {
  try {
    const manifest = yaml.safeLoad(fs.readFileSync(manifestPath));
    const reqKeys = ['name', 'description'];

    for (const key of reqKeys) {
      if (!manifest[key]) {
        console.error(`Key ${chalk.cyan(key)} is missing in manifest.yaml`);
        process.exit(1);
      }
    }
  } catch (e) {
    console.error('No manifest file found');
    process.exit(1);
  }
}

function checkStaticFiles() {
  const reqEntries = ['asset-manifest.json', 'static'];

  for (const reqEntry of reqEntries) {
    const file = path.resolve(staticFilesPath, reqEntry);

    if (!fs.existsSync(file)) {
      const kind = reqEntry.includes('.') ? 'File' : 'Directory';

      console.error(
        `${kind} ${chalk.cyan(reqEntry)} was not found in ${chalk.green(
          'build',
        )} directory`,
      );
      process.exit(1);
    }
  }
}

function bundleApp() {
  const output = fs.createWriteStream(bundlePath);
  const archive = archiver('zip');

  output.on('close', () => {
    console.log(
      `Success! Application has been bundled to ${chalk.green('bundle.zip')}`,
    );
    console.log(
      `Run ${chalk.cyan(
        'hiro-sdk publish',
      )} to publish application to HIRO Desktop`,
    );
  });

  archive.on('warning', function (err) {
    console.log('Error occured', err);
    process.exit(1);
  });

  archive.on('error', function (err) {
    console.log('Error occured', err);
    process.exit(1);
  });

  archive.pipe(output);

  archive.file('manifest.yaml');
  archive.append(null, { name: 'content', type: 'directory' });
  archive.directory('build/', 'content');

  archive.finalize();
}
