#!/usr/bin/env node

const path = require('path');
const fs = require('fs');
const https = require('https');

const _ = require('lodash');
const axios = require('axios').default;
const yaml = require('js-yaml');
const chalk = require('chalk');
const program = require('commander');
const HiroClient = require('@hiro-graph/client').default;
const dotenv = require('dotenv');

const ORG_ID_GRAPHIT_CO = 'cju16o7cg0001mz775fnoq66i_cjnu3p5tf001bs934wf10n26n'; // Hardcoded in the hiro_admin tool also

program
  .option('-h, --host <host>', 'GraphIT host')
  .option('-t, --token <token>', 'Authorization token')
  .option('-p, --parent <parent>', 'ogit/Auth/Application/parent value')
  .option('-o, --org <org>', 'ogit/_organization value')
  .option('--public', 'Make new application public', false)
  .option('-e, --env', 'Read host and token from the dotenv files')
  .on('--help', onHelp)
  .parse(process.argv);

let { host, token, parent, public: isPublic, org } = program;

const manifest = loadManifest();

if (program.env) {
  const files = [
    '.env',
    '.env.local',
    '.env.development',
    '.env.development.local',
  ];
  const envVars = process.env;

  for (const file of files) {
    const result = dotenv.load({ path: file });

    _.assign(envVars, result.parsed);
  }

  if (envVars.HIRO_GRAPH_URL) {
    host = envVars.HIRO_GRAPH_URL;
  }

  if (envVars.HIRO_GRAPH_TOKEN) {
    token = envVars.HIRO_GRAPH_TOKEN;
  }

  if (envVars.HIRO_DESKTOP_ID) {
    parent = envVars.HIRO_DESKTOP_ID;
  }

  if (envVars.HIRO_ORGANIZATION) {
    org = envVars.HIRO_ORGANIZATION;
  }

  if (envVars.HIRO_PUBLIC) {
    isPublic = envVars.HIRO_PUBLIC;
  }
}

if (!host || !token || !parent || (!org && !isPublic)) {
  console.log(
    'You have not set one of these flags: host, token, parent. Also, please check that you provided one of these values: --org or --public',
  );
  program.outputHelp();
  process.exit(1);
}

function onHelp() {
  console.log();
  console.log(chalk.yellow('  USAGE:'));
  console.log(
    'Public applications can be published with the --public flag:',
    chalk.cyan(
      '  hiro-sdk publish -h http://graphi.co -t 28m90v5v825m2v2 -p v5mu293v523v52 --public',
    ),
    'Alternatively, an organisation can be set:',
    chalk.cyan(
      '  hiro-sdk publish -h http://graphi.co -t 28m90v5v825m2v2 -p v5mu293v523v52 -o sdf345gdfhg345',
    ),
  );
  console.log(
    `  Publish bundled application by providing host and token explicitly.`,
  );
  console.log();
  console.log(chalk.cyan('  hiro-sdk publish -e'));
  console.log(
    '  Publish bundled application by reading host and token from dotenv files or the current environment',
  );
  console.log(
    `  Supported files:  ${chalk.cyan('.env')}, ${chalk.cyan(
      '.env.local',
    )}, ${chalk.cyan('.env.development')}, ${chalk.cyan(
      '.env.development.local',
    )}`,
  );
  console.log(`  Host will be read from ${chalk.cyan('HIRO_GRAPH_URL')},
    token from ${chalk.cyan('HIRO_GRAPH_TOKEN')},
    parent from ${chalk.cyan('HIRO_DESKTOP_ID')},
    org from ${chalk.cyan('HIRO_ORGANIZATION')},
    public from ${chalk.cyan('HIRO_PUBLIC')}`);
}

const http = createAxiosClient();
const appName = getValueFromManifest('name');

console.log('Searching for desktop application', chalk.cyan(appName));

getClient()
  .lucene(
    `+ogit\\/_type:"ogit/Auth/Application" +ogit\\/Auth\\/Application\\/type:"desktop" +ogit\\/Auth\\/Application\\/status:"active"  +ogit\\/name:"${appName}"`,
    { limit: 1 },
  )
  .then(findOrCreate)
  .then(updateContent)
  .then(activateApp)
  .then(() => {
    console.log();
    console.log('Application has been published successfully!');
  })
  .catch(logErrorAndExit);

function findOrCreate(result) {
  if (!result || result.length === 0) {
    return createNew();
  }

  console.log('Found existing application:', result[0]);

  return result[0]['ogit/_id'];
}

function createNew() {
  console.log('Application does not exist.');
  console.log('Creating application...');

  const appRules = getApplicationRules();
  const orgId = isPublic ? ORG_ID_GRAPHIT_CO : org;

  const appEntity = {
    'ogit/name': getValueFromManifest('name'),
    'ogit/description': getValueFromManifest('description'),
    'ogit/Auth/Application/parent': parent,
    'ogit/Auth/vertexRule': appRules.vertexRule,
    'ogit/Auth/edgeRule': appRules.edgeRule,
    'ogit/_organization': orgId,
  };

  return http.post('/desktop', appEntity).then((response) => {
    console.log('Created application:', response.data);

    return response.data['ogit/_id'];
  });
}

function updateContent(appId) {
  console.log('Updating application content...');

  return http
    .put(`/${appId}/content`, getBundleStream(), {
      headers: { 'Content-Type': 'application/json' },
    })
    .then(() => {
      console.log('Updated application content', appId);

      return appId;
    });
}

function activateApp(appId) {
  console.log('Activating an application...');

  return http.patch(`/${appId}/content`, {}).then((response) => {
    console.log('Activated. Generated credentials:', response.data);
  });
}

function logErrorAndExit(error) {
  console.log();
  console.error('Error encountered!');
  console.log(_.get(error, 'response.data.error', error));
  process.exit(1);
}

function createAxiosClient() {
  const agent = new https.Agent({
    rejectUnauthorized: false,
  });

  return axios.create({
    baseURL: _.trimEnd(host, '/') + '/api/app-admin/1',
    headers: {
      Authorization: `Bearer ${token}`,
    },
    httpsAgent: agent,
  });
}

function getClient() {
  const hiroClient = new HiroClient(
    {
      endpoint: host,
      token: token,
    },
    {
      forceHTTP: true,
    },
  );

  return hiroClient;
}

function loadManifest() {
  try {
    return yaml.safeLoad(fs.readFileSync(path.resolve('manifest.yaml')));
  } catch (e) {
    console.error('No manifest file found');
    process.exit(1);
  }
}

function getValueFromManifest(key) {
  if (manifest[key]) {
    return manifest[key];
  }

  console.error(
    `No ${chalk.cyan(key)} key found in ${chalk.green('manifest.yaml')}`,
  );
  process.exit(1);
}

function getBundleStream() {
  try {
    return fs.createReadStream(path.resolve('bundle.zip'));
  } catch (e) {
    console.error(
      `Can not create a readable stream from the ${chalk.cyan('bundle.zip')}`,
    );
    process.exit(1);
  }
}

function getApplicationRules() {
  try {
    const appRules = require(path.resolve('./application.json'));

    const {
      'ogit/Auth/vertexRule': vertexRule,
      'ogit/Auth/edgeRule': edgeRule,
    } = appRules;

    return { vertexRule, edgeRule };
  } catch (e) {
    console.error('No application.json file found. Exiting...');
    process.exit(1);
  }
}
